class
	DEMO_WINDOW

inherit
	EV_TITLED_WINDOW

create
	make

feature {NONE}

	make
		local
			l_table: EV_TABLE
		do
			make_with_title ("Arbitrary-Precision Integer Calculator")

			create number_field
			number_field.disable_edit
			number_field.key_press_actions.extend (agent key_pressed)

			create l_table
			l_table.resize (4, 6)
			l_table.add (number_field, 1, 1, 4, 1)
			l_table.add (create {EV_BUTTON}.make_with_text_and_action (" + ", agent action_pressed ('+')), 1, 2, 1, 1)
			l_table.add (create {EV_BUTTON}.make_with_text_and_action (" - ", agent action_pressed ('-')), 2, 2, 1, 1)
			l_table.add (create {EV_BUTTON}.make_with_text_and_action (" * ", agent action_pressed ('*')), 3, 2, 1, 1)
			l_table.add (create {EV_BUTTON}.make_with_text_and_action (" / ", agent action_pressed ('/')), 4, 2, 1, 1)
			l_table.add (create {EV_BUTTON}.make_with_text_and_action (" 7 ", agent number_pressed (7)), 1, 3, 1, 1)
			l_table.add (create {EV_BUTTON}.make_with_text_and_action (" 8 ", agent number_pressed (8)), 2, 3, 1, 1)
			l_table.add (create {EV_BUTTON}.make_with_text_and_action (" 9 ", agent number_pressed (9)), 3, 3, 1, 1)
			l_table.add (create {EV_BUTTON}.make_with_text_and_action (" ^ ", agent action_pressed ('^')), 4, 3, 1, 1)
			l_table.add (create {EV_BUTTON}.make_with_text_and_action (" 4 ", agent number_pressed (4)), 1, 4, 1, 1)
			l_table.add (create {EV_BUTTON}.make_with_text_and_action (" 5 ", agent number_pressed (5)), 2, 4, 1, 1)
			l_table.add (create {EV_BUTTON}.make_with_text_and_action (" 6 ", agent number_pressed (6)), 3, 4, 1, 1)
			l_table.add (create {EV_BUTTON}.make_with_text_and_action ("+/-", agent opposite_pressed), 4, 4, 1, 1)
			l_table.add (create {EV_BUTTON}.make_with_text_and_action (" 1 ", agent number_pressed (1)), 1, 5, 1, 1)
			l_table.add (create {EV_BUTTON}.make_with_text_and_action (" 2 ", agent number_pressed (2)), 2, 5, 1, 1)
			l_table.add (create {EV_BUTTON}.make_with_text_and_action (" 3 ", agent number_pressed (3)), 3, 5, 1, 1)
			l_table.add (create {EV_BUTTON}.make_with_text_and_action (" C ", agent clear_pressed), 4, 5, 1, 1)
			l_table.add (create {EV_BUTTON}.make_with_text_and_action (" 0 ", agent number_pressed (0)), 1, 6, 3, 1)
			l_table.add (create {EV_BUTTON}.make_with_text_and_action (" = ", agent action_pressed ('=')), 4, 6, 1, 1)

			extend (l_table)
			close_request_actions.extend (agent close_clicked)

			create stack.make
			clear_pressed
		end

feature {NONE}

	number_field: EV_TEXT_FIELD

	number: MPZ

	stack: TWO_WAY_LIST [TUPLE [number: MPZ; action: CHARACTER_8]]

feature {NONE}

	number_pressed (a_number: NATURAL_32)
		local
			l_zero: MPZ
		do
			if number >= l_zero then
				set_number (number |*| 10 |+| a_number)
			else
				set_number (number |*| 10 |-| a_number)
			end
		end

	opposite_pressed
		do
			set_number (-number)
		end

	action_pressed (a_action: CHARACTER_8)
		local
			l_zero: MPZ
			l_cur: MPZ
			l_action: CHARACTER_8
			l_not_supported: BOOLEAN
		do
			inspect a_action
			when '+', '-', '*', '/', '^' then
				stack.extend ([number, a_action])
				set_number (l_zero)
			when '=' then
				stack.extend ([number, '+'])
				across
					stack as c
				until
					l_not_supported
				loop
					if c.target_index = 1 then
						l_cur := c.item.number
					else
						inspect l_action
						when '+' then l_cur := l_cur + c.item.number
						when '-' then l_cur := l_cur - c.item.number
						when '*' then l_cur := l_cur * c.item.number
						when '/' then
							if c.item.number /= l_zero then
								l_cur := l_cur / c.item.number
							else
								l_not_supported := True
							end
						when '^' then
							if c.item.number.fits_into_natural_32 then
								l_cur := l_cur ^ c.item.number.to_natural_32
							else
								l_not_supported := True
							end
						end
					end
					l_action := c.item.action
				end
				stack.wipe_out
				set_number (l_cur)
			end
			if l_not_supported then
				clear_pressed
				number_field.set_text ("Not supported...")
			end
		end

	clear_pressed
		local
			l_zero: MPZ
		do
			stack.wipe_out
			set_number (l_zero)
		end

	key_pressed (a_key: EV_KEY)
		local
			l_zero, l_tmp: MPZ
		do
			inspect a_key.code
			when {EV_KEY_CONSTANTS}.Key_numpad_0, {EV_KEY_CONSTANTS}.Key_0 then number_pressed (0)
			when {EV_KEY_CONSTANTS}.Key_numpad_1, {EV_KEY_CONSTANTS}.Key_1 then number_pressed (1)
			when {EV_KEY_CONSTANTS}.Key_numpad_2, {EV_KEY_CONSTANTS}.Key_2 then number_pressed (2)
			when {EV_KEY_CONSTANTS}.Key_numpad_3, {EV_KEY_CONSTANTS}.Key_3 then number_pressed (3)
			when {EV_KEY_CONSTANTS}.Key_numpad_4, {EV_KEY_CONSTANTS}.Key_4 then number_pressed (4)
			when {EV_KEY_CONSTANTS}.Key_numpad_5, {EV_KEY_CONSTANTS}.Key_5 then number_pressed (5)
			when {EV_KEY_CONSTANTS}.Key_numpad_6, {EV_KEY_CONSTANTS}.Key_6 then number_pressed (6)
			when {EV_KEY_CONSTANTS}.Key_numpad_7, {EV_KEY_CONSTANTS}.Key_7 then number_pressed (7)
			when {EV_KEY_CONSTANTS}.Key_numpad_8, {EV_KEY_CONSTANTS}.Key_8 then number_pressed (8)
			when {EV_KEY_CONSTANTS}.Key_numpad_9, {EV_KEY_CONSTANTS}.Key_9 then number_pressed (9)
			when {EV_KEY_CONSTANTS}.Key_numpad_add      then action_pressed ('+')
			when {EV_KEY_CONSTANTS}.Key_numpad_subtract then action_pressed ('-')
			when {EV_KEY_CONSTANTS}.Key_numpad_multiply then action_pressed ('*')
			when {EV_KEY_CONSTANTS}.Key_numpad_divide   then action_pressed ('/')
			when {EV_KEY_CONSTANTS}.Key_enter           then action_pressed ('=')
			when {EV_KEY_CONSTANTS}.Key_back_space then
				if number /= l_zero then
					set_number (number |/| 10)
				elseif not stack.is_empty then
					stack.finish
					l_tmp := stack.item.number
					stack.remove
					set_number (l_tmp)
				end
			else
				-- ignore
			end
		end

	close_clicked
		do
			destroy
			environment_i.application.destroy
		end

feature {NONE}

	set_number (a_number: MPZ)
		local
			l_s: STRING_8
			l_zero: MPZ
		do
			create l_s.make_empty
			across stack as c loop
				l_s.append (c.item.number.out)
				l_s.append (c.item.action.out)
			end
			if a_number /= l_zero then
				l_s.append (a_number.out)
			end

			number := a_number
			number_field.set_text (l_s)

			-- Don't allow the focus to be on a button.
			-- "Enter" would press it otherwise.
			number_field.set_focus
			number_field.set_caret_position (l_s.count + 1)
		end

invariant
	number_field /= Void
	stack /= Void

end
